package jumpstart.max.business.domain.base;

import java.io.Serializable;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import jumpstart.max.business.commons.exception.CannotDeleteIsReferencedException;
import jumpstart.max.business.commons.exception.DoesNotExistException;
import jumpstart.max.business.commons.exception.DuplicateAlternateKeyException;
import jumpstart.max.business.commons.exception.DuplicatePrimaryKeyException;
import jumpstart.max.business.commons.exception.OptimisticLockException;
import jumpstart.max.business.commons.exception.ServiceException;
import jumpstart.max.business.commons.interpreter.IEJBExceptionInterpreter;
import jumpstart.max.business.commons.interpreter.JBossExceptionInterpreter;

public abstract class BaseSvc {

	@PersistenceContext(unitName = "jumpstart-max")
	protected EntityManager _em;

	protected IEJBExceptionInterpreter _exceptionInterpreter = new JBossExceptionInterpreter();

	protected Object find(Class<?> cls, Serializable id) throws DoesNotExistException {

		if (id == null) {
			throw new IllegalArgumentException("Null id has been given to find " + cls.getName() + ".");
		}
		else if (id.equals(0)) {
			throw new IllegalArgumentException("Empty id has been given to find " + cls.getName() + ".");
		}

		try {
			Object obj = _em.find(cls, id);

			if (obj == null) {
				throw new DoesNotExistException(cls, id);
			}

			return obj;
		}
		catch (IllegalArgumentException e) {
			// Invalid OID
			throw new IllegalArgumentException("Invalid id has been given to find object of type " + cls.getName()
					+ ".  Id given is \"" + id + "\"", e);
		}
		// todo - review this
		catch (Exception e) {
			// Doesn't exist
			throw new DoesNotExistException(cls, id);
		}
	}

	public void persist(BaseEntity entity) throws ServiceException {
		try {
			_em.persist(entity);
		}
		catch (RuntimeException e) {
			handlePersistenceException(e, entity, entity.getIdForMessages());
			throw new IllegalStateException("Shouldn't get here");
		}
	}

	public BaseEntity merge(BaseEntity entity) throws ServiceException {
		try {
			entity = _em.merge(entity);
		}
		catch (RuntimeException e) {
			handlePersistenceException(e, entity, entity.getIdForMessages());
			throw new IllegalStateException("Shouldn't get here");
		}
		return entity;
	}

	public void remove(BaseEntity entity) throws ServiceException {
		try {
			_em.remove(entity);
		}
		catch (RuntimeException e) {
			handlePersistenceException(e, entity, entity.getIdForMessages());
			throw new IllegalStateException("Shouldn't get here");
		}
	}

	protected void handlePersistenceException(RuntimeException e, BaseEntity entity, Serializable id)
			throws ServiceException {

		// TODO - review later because this is JBoss & hibernate specific, which
		// I hope it won't be in final EJB3.

		if (e instanceof javax.persistence.OptimisticLockException) {
			throw new OptimisticLockException(entity, id, e);
		}
		else {
			ServiceException s = _exceptionInterpreter.interpret(e);

			if (s instanceof DuplicatePrimaryKeyException) {
				s = new DuplicatePrimaryKeyException(entity, id);
			}
			else if (s instanceof DuplicateAlternateKeyException) {
				s = new DuplicateAlternateKeyException(entity, s.getMessage());
			}
			else if (s instanceof CannotDeleteIsReferencedException) {
				CannotDeleteIsReferencedException c = (CannotDeleteIsReferencedException) s;
				s = new CannotDeleteIsReferencedException(entity, id, c.getReferencedByEntityName(), c
						.getReferencedByPropertyName());
			}

			throw s;
		}
	}

}
