package jumpstart.max.web.state;

import java.io.Serializable;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;

import jumpstart.max.business.domain.security.User;
import jumpstart.max.business.domain.security.UserRole;
import jumpstart.max.web.commons.translator.DateMidnightTranslator;

import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

@SuppressWarnings("serial")
public class Visit implements Serializable {

	private boolean _loggedIn = false;

	// The logged in user
	// - Note this is a security risk because User holds a clear-text password.
	// Solutions include keeping password encrypted in User; moving password
	// out of User into a separate entity; using third-party authentication;
	// holding only those fields from User that we'll use in here.
	private User _myUser = null;
	private List<Long> _myActiveRoleIds = null;

	// Date translator - for inputting dates
	private DateMidnightTranslator _dateMidnightTranslator;

	// Date and time view formatters - for outputting dates
	private DateTimeFormatter _dateViewFormatter;
	private DateTimeFormatter _time4ViewFormatter;
	private DateTimeFormatter _time6ViewFormatter;
	private DateTimeFormatter _dateTime4ViewFormatter;
	private DateTimeFormatter _dateTime6ViewFormatter;

	// Date and time list formatters - for outputting dates in a list
	private DateTimeFormatter _dateListFormatter;
	private DateTimeFormatter _time4ListFormatter;
	private DateTimeFormatter _time6ListFormatter;
	private DateTimeFormatter _dateTime4ListFormatter;
	private DateTimeFormatter _dateTime6ListFormatter;

	//
	// The User
	//

	public void noteLogIn(User user) {
		_loggedIn = true;
		_myActiveRoleIds = null;

		noteChanges(user);
	}

	public void noteLogOut() {
		_loggedIn = false;
		_myActiveRoleIds = null;
		
		_myUser = null;
	}

	public void noteChanges(User user) {

		if (user == null) {
			throw new IllegalArgumentException();
		}
		
		_myUser = user;

		_dateMidnightTranslator = new DateMidnightTranslator();
		_dateMidnightTranslator.setPattern(user.getDateInputPattern());

		_dateViewFormatter = DateTimeFormat.forPattern(user.getDateViewPattern());
		_time4ViewFormatter = DateTimeFormat.forPattern(user.getTime4ViewPattern());
		_time6ViewFormatter = DateTimeFormat.forPattern(user.getTime6ViewPattern());
		_dateTime4ViewFormatter = DateTimeFormat.forPattern(user.getDateTime4ViewPattern());
		_dateTime6ViewFormatter = DateTimeFormat.forPattern(user.getDateTime6ViewPattern());

		_dateListFormatter = DateTimeFormat.forPattern(user.getDateListPattern());
		_time4ListFormatter = DateTimeFormat.forPattern(user.getTime4ListPattern());
		_time6ListFormatter = DateTimeFormat.forPattern(user.getTime6ListPattern());
		_dateTime4ListFormatter = DateTimeFormat.forPattern(user.getDateTime4ListPattern());
		_dateTime6ListFormatter = DateTimeFormat.forPattern(user.getDateTime6ListPattern());
	}

	public boolean isLoggedIn() {
		return _loggedIn;
	}

	public Long getMyUserId() {
		return _myUser.getId();
	}

	public String getMyLoginId() {
		return _myUser.getLoginId();
	}

	//
	// The User's roles
	// These are lazy loaded by ProtectedBasePage
	//

	public List<Long> getMyActiveRoleIds() {
		return _myActiveRoleIds;
	}

	public void setMyUserRoles(List<UserRole> userRoles) {
		_myActiveRoleIds = new ArrayList<Long>();

		// Add the active userRoles to my list

		for (UserRole userRole : userRoles) {
			if (userRole.getStatus().getId().equals(UserRole.STATUS_ACTIVE)) {
				_myActiveRoleIds.add(userRole.getRole().getId());
			}
		}
	}

	//
	// Date and time formatting
	//

	public DateMidnightTranslator getDateMidnightTranslator() {
		return _dateMidnightTranslator;
	}

	public DateTimeFormatter getDateViewFormatter() {
		return _dateViewFormatter;
	}

	public DateTimeFormatter getDateListFormatter() {
		return _dateListFormatter;
	}

	public DateTimeFormatter getTimeViewFormatter(int timePrecision) {
		if (timePrecision == 4) {
			return _time4ViewFormatter;
		}
		else {
			return _time6ViewFormatter;
		}
	}

	public DateTimeFormatter getTimeListFormatter(int timePrecision) {
		if (timePrecision == 4) {
			return _time4ListFormatter;
		}
		else {
			return _time6ListFormatter;
		}
	}

	public DateTimeFormatter getDateTimeViewFormatter(int timePrecision) {
		if (timePrecision == 4) {
			return _dateTime4ViewFormatter;
		}
		else {
			return _dateTime6ViewFormatter;
		}
	}

	public DateTimeFormatter getDateTimeListFormatter(int timePrecision) {
		if (timePrecision == 4) {
			return _dateTime4ListFormatter;
		}
		else {
			return _dateTime6ListFormatter;
		}
	}

	//
	// Currency formatting
	//

	public NumberFormat getCurrencyFormat() {
		NumberFormat format = NumberFormat.getInstance();
		format.setMaximumFractionDigits(2);
		format.setMinimumFractionDigits(2);
		return format;
	}

	//
	// Stylesheet
	//

	public int getStyleId() {
		return _myUser.getStyleId();
	}

	//
	// Tapestry Inspector
	// Is of limited value for now due to a bug to be fixed in 4.1.1: See
	// http://www.nabble.com/-jira--Created:-(TAPESTRY-1050)-The-InspectorButton-doesn't-work:-t2062427.html
	//

	public boolean isInspectorEnabled() {
		return _myUser.isShowInspector();
	}

}
